#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""  
Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors

SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import logging, time
import subprocess
from c8ydm.framework.modulebase import Sensor, Initializer
from c8ydm.framework.smartrest import SmartRESTMessage
from c8ydm.core.device_stats import DeviceStats

class DeviceSensor(Sensor, Initializer):
    logger = logging.getLogger(__name__)
    DeviceStats = DeviceStats()

    def getSensorMessages(self):
        try:
            return self.sendStats()
        except Exception as e:
            self.logger.exception(f'Error in DeviceSensor getSensorMessages: {e}', e)

    def getMessages(self):
        try:
            return self.sendStats()
        except Exception as e:
            self.logger.exception(f'Error in DeviceSensor getMessages: {e}', e)
    
    def sendStats(self):
        self.stats = []
        for key,value in self._getCPU().items():
            self.stats.append(SmartRESTMessage('s/us', '200', ['cpu', key, value]))
        for key,value in self._getDisk().items():
            self.stats.append(SmartRESTMessage('s/us', '200', ['disk', key, value]))
        for key,value in self._getMemory().items():
            self.stats.append(SmartRESTMessage('s/us', '200', ['memory', key, value]))
        return self.stats
    
    def _getCPU(self):
        return self.DeviceStats.getCPUStats() 

    def _getDisk(self):
        return self.DeviceStats.getDiskStats() 
    
    def _getMemory(self):
        return self.DeviceStats.getMemoryStats()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""  
Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors

SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import logging, io, re
from posixpath import dirname
from datetime import datetime
from c8ydm.framework.modulebase import Initializer, Listener
from c8ydm.framework.smartrest import SmartRESTMessage
from os.path import expanduser,exists,dirname
import pathlib
import subprocess

class DownloadConfigfileInitializer(Initializer, Listener):
    logger = logging.getLogger(__name__)
    fragment = 'c8y_DownloadConfigFile'
    

    def getMessages(self):
        return []

    def getSupportedOperations(self):
        return ['c8y_DownloadConfigFile']
    
    def getSupportedTemplates(self):
        return []

    def _set_executing(self):
        executing = SmartRESTMessage('s/us', '501', [self.fragment])
        print("Executing MSG send")
        self.agent.publishMessage(executing)

    #datei anhängen
    def _set_success(self, url):
        success = SmartRESTMessage('s/us', '503', [self.fragment, url])
        print("Success MSG send")
        self.agent.publishMessage(success)

    def _set_failed(self, reason):
        failed = SmartRESTMessage('s/us', '502', [self.fragment,reason])
        self.logger.error(f'Operation failed, reason: {reason}')
        self.agent.publishMessage(failed)
    
    def handleOperation(self, message):
        mo_id = self.agent.rest_client.get_internal_id(self.agent.serial)
        home = expanduser('~')
        root = pathlib.Path(home + '/.cumulocity')
        configfiles = {'sshd': '/etc/ssh/sshd_config', 'agent': f'{root}/agent.ini'}  
        try:
            if 's/ds' in message.topic and message.messageId == '524':
                deviceid = message.values[0]
                binaryurl = message.values[1]
                configtype = message.values[2]
                self._set_executing()
                if 'cumulocity' in binaryurl:  
                    if configtype in configfiles:
                        path = pathlib.Path(configfiles[configtype])
                        self.logger.info(dirname(path))
                        if pathlib.Path(dirname(configfiles[configtype])).exists():
                            process = subprocess.Popen(["cp",str(path),f'{str(path)}_backup'],stdout=subprocess.PIPE,stderr=subprocess.PIPE)
                            process.wait()
                            if self.agent.rest_client.download_c8y_binary(binaryurl,str(path)) is not None:
                                eventMsg = SmartRESTMessage('s/us', '400', ['c8y_ConfigDownloadEvent', f'Config {configtype} was downloaded to {str(path)}. Backup of old config file was created.'])
                                self.agent.publishMessage(eventMsg)
                                self._set_success(binaryurl)
                            else:
                                self._set_failed("Failed to download file from c8y binary")
                        else:
                            self._set_failed("Directory of config file does not exist")
                    else:
                        self._set_failed("Do not know config file type")
                else:
                    self._set_failed("Currently only c8y binary supported")
            self.logger.debug("download configfile handled")
        except Exception as e:
            self._set_failed(e)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""  
Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors

SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from abc import ABCMeta, abstractmethod

class Sensor:
  __metaclass__ = ABCMeta

  def __init__(self, serial, agent):
    self.serial = serial
    self.agent = agent

  '''
  Returns a list of SmartREST messages. Will be called every iteration of the main loop.
  '''
  @abstractmethod
  def getSensorMessages(self): pass

class Listener:
  __metaclass__ = ABCMeta

  def __init__(self, serial, agent):
    self.serial = serial
    self.agent = agent

  '''
  Callback that is executed for any operation received
  '''
  @abstractmethod
  def handleOperation(self, message): pass

  '''
  Returns a list of supported operations
  '''
  @abstractmethod
  def getSupportedOperations(self): pass

  '''
  Returns a list of supported SmartREST templates (X-Ids)
  '''
  @abstractmethod
  def getSupportedTemplates(self): pass

class Initializer:
  __metaclass__ = ABCMeta

  def __init__(self, serial, agent):
    self.serial = serial
    self.agent = agent
  '''
  Returns a list of SmartREST messages. Will be called at the start of the agent
  '''
  @abstractmethod
  def getMessages(self): pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors

SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import logging
import platform
import distro
from c8ydm.framework.smartrest import SmartRESTMessage

if 'Linux' == platform.system() and distro.id() in ['debian','ubuntu','raspbian']:
    import apt
else:
    apt = None

class AptPackageManager:
    logger = logging.getLogger(__name__)

    def getInstalledSoftware(self, with_update):
        allInstalled = []
        if apt:
            cache = apt.cache.Cache()
            if with_update:
                self.logger.info('Starting apt update....')
                cache.update()
                self.logger.info('apt update finished!')
            cache.open()

            for pkg in cache:
                if (pkg.is_installed and not pkg.shortname.startswith('lib') and not pkg.shortname.startswith('python')):
                    allInstalled.append(pkg.shortname)
                    allInstalled.append(pkg.installed.version)
                    allInstalled.append('')

            cache.close()

        return SmartRESTMessage('s/us', '116', allInstalled)
    
    def get_installed_software_json(self, with_update):
        software_list = []
        all_installed = {
            "c8y_SoftwareList": software_list
        }
        if apt:
            cache = apt.cache.Cache()
            if with_update:
                self.logger.info('Starting apt update....')
                cache.update()
                self.logger.info('apt update finished!')
            cache.open()
            for pkg in cache:
                if (pkg.is_installed):
                    software = {
                        "name": pkg.shortname,
			            "version": pkg.installed.version,
			            "url": ""
                    }
                    software_list.append(software)
            cache.close()
        return all_installed

    
    def install_software(self, software_to_install, with_update):
        errors = []
        if apt:
            cache = apt.cache.Cache()
            if with_update:
                self.logger.info('Starting apt update....')
                cache.update()
                self.logger.info('apt update finished!')
            cache.open()
            for software in software_to_install:
                name = software[0]
                version = software[1]
                # url = software[2]
                action = software[3]
                pkg = cache[name]
                if pkg is None:
                    errors.append('No Software found with name '+ name)
                else:
                    if action == 'install':
                        if version == 'latest':
                            self.logger.info('install ' + pkg.shortname + '=latest')
                            pkg.mark_install()
                        else:
                            self.logger.info(
                                'install ' + pkg.shortname + '=' + version)
                            candidate = pkg.versions.get(version)
                            if candidate == None:
                                errors.append('Version '+ version +' not available in Repo!')
                            else:
                                pkg.candidate = candidate
                                pkg.mark_install()
                    # Software currently installed in the same version
                    if action == 'update' and pkg.is_installed and pkg.installed.version == version:
                        # no action needed
                        self.logger.debug('existing ' + pkg.shortname +
                                        '=' + pkg.installed.version)
                    if action == 'update' and pkg.is_installed and pkg.installed.version != version:
                        self.logger.info('install ' + pkg.shortname + '=' + version)
                        candidate = pkg.versions.get(version)
                        if candidate == None:
                                errors.append('Version '+ version +' not available in Repo!')
                        else:
                            pkg.candidate = candidate
                            pkg.mark_install()
                    if action == 'delete' and pkg.is_installed:
                        self.logger.info('delete ' + pkg.shortname +
                                        '=' + pkg.installed.version)
                        pkg.mark_delete()
            try:
                self.logger.info('Starting apt install/removal of Software..')
                cache.commit()
                self.logger.info("Install/Removal of Software finished!")
            except Exception as e:
                self.logger.error(e)

        return errors

    """ Old Deprecated Version of Software Updates """
    def installSoftware(self, toBeInstalled, with_update):
        cache = apt.cache.Cache()
        if with_update:
            self.logger.info('Starting apt update....')
            cache.update()
            self.logger.info('apt update finished!')
        cache.open()

        for software in toBeInstalled:
            pkg = cache[software[0]]
            # Software currently installed in the same version
            if pkg.is_installed and pkg.installed.version == software[1]:
                # no action needed
                self.logger.debug('existing ' + pkg.shortname +
                                    '=' + pkg.installed.version)
            else:
                self.logger.info(
                    'install ' + pkg.shortname + '=' + software[1])
                candidate = pkg.versions.get(software[1])
                pkg.candidate = candidate
                pkg.mark_install()

        # Check what needs to be uninstalled
        toBeInstalledSoftware = [i[0] for i in toBeInstalled]
        for pkg in cache:
            if not pkg.shortname.startswith('lib') and pkg.is_installed and pkg.shortname not in toBeInstalledSoftware:
                self.logger.info('delete ' + pkg.shortname +
                                    '=' + pkg.installed.version)
                pkg.mark_delete()

        try:
            self.logger.info('Starting apt install/removal of Software..')
            cache.commit()
            self.logger.info("Install/Removal of Software finished!")
        except Exception as e:
            self.logger.error(e)

        return []



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""  
Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors

SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import logging, time
import psutil



class DeviceStats:
    logger = logging.getLogger(__name__)
    
    def __init__(self):
        pass

    def getMemoryStats(self):
        try:
            memory = {}
            memory['free'] = psutil.virtual_memory().free
            memory['used'] = psutil.virtual_memory().used
            memory['total'] = psutil.virtual_memory().total
            memory['percent'] = psutil.virtual_memory().percent
            self.logger.debug("Collected the following memory stats: %s" % (memory))
        except Exception as e:
            self.logger.error('The following error occured: %s' % (str(e)))
        finally:
            return memory

    def getCPUStats(self):
        try:
            cpu = {}
            cpu['guest'] = psutil.cpu_times_percent(interval=1, percpu=False)[0]
            cpu['idle'] = psutil.cpu_times_percent(interval=1, percpu=False)[2]
            cpu['iowait'] = psutil.cpu_times_percent(interval=1, percpu=False)[3]
            cpu['irq'] = psutil.cpu_times_percent(interval=1, percpu=False)[4]
            cpu['system'] = psutil.cpu_times_percent(interval=1, percpu=False)[8]
            cpu['user'] = psutil.cpu_times_percent(interval=1, percpu=False)[9]
            self.logger.debug("Collected the following cpu stats: %s" % (cpu))
        except Exception as e:
            self.logger.error('The following error occured: %s' % (str(e)))
        finally:
            return cpu

    def getDiskStats(self):
        try:
            disk = {}
            disk['total'] = psutil.disk_usage('/').total
            disk['used'] = psutil.disk_usage('/').used
            disk['free'] = psutil.disk_usage('/').free
            disk['percent'] = psutil.disk_usage('/').percent
            self.logger.debug("Collected the following disk stats: %s" % (disk))
        except Exception as e:
            self.logger.error('The following error occured: %s' % (str(e)))
        finally:
            return disk


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""  
Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors

SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import logging, time, uuid
import socket
import ipaddress
from requests import get
from c8ydm.framework.modulebase import Initializer
from c8ydm.framework.smartrest import SmartRESTMessage
from c8ydm.core.geo_position_resolver import GeoPositionResolver

class Network(Initializer):
    logger = logging.getLogger(__name__)
    net_message_id = 'dm100'
    pos_message_id = '402'
    xid = 'c8y-dm-agent-v1.0'
    def_adapter = None
    geo_pos_resolver = GeoPositionResolver()

    def getMessages(self):
        net_msg = None
        pos_msg = None
        try:
            self.logger.info(f'Network Initializer called...')
            ip = socket.gethostbyname(socket.gethostname())
            netmask = ipaddress.IPv4Network(ip).netmask
            name = None
            mac = self.get_mac()
            for interface in socket.if_nameindex():
                if_name = interface[1]
                if  if_name.startswith('eth'):
                    name = interface[1]
                    break
            enabled = 1
            net_msg = SmartRESTMessage('s/uc/'+self.xid, self.net_message_id, [self.serial, ip, netmask, name, enabled, mac])
            pub_ip = self.get_public_ip()
            lat_lng = self.geo_pos_resolver.get_pos_by_ip(pub_ip)
            
            if lat_lng and lat_lng['lat'] is not None and lat_lng['lng'] is not None:
                pos_msg = SmartRESTMessage('s/us', self.pos_message_id, [lat_lng['lat'], lat_lng['lng']])
            return [net_msg, pos_msg]
        except Exception as ex:
            self.logger.error(f'Error on retrieving Network Details: {ex}')
            return None
        
    
    def get_public_ip(self):
        try:
            ip = get('https://api.ipify.org').text
            self.logger.debug(f'Public IP: {ip}')
            return ip
        except Exception as ex:
            self.logger.error(f'Error retrieving public IP: {ex}')
            return None
    
    def get_mac(self):
        mac_num = hex(uuid.getnode()).replace('0x', '').replace('L', '').upper()
        mac_num = mac_num.zfill(12)
        mac = ':'.join(mac_num[i: i + 2] for i in range(0, 11, 2))
        return mac





#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""  
Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors

SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import logging
import re
import sys
from typing import List, Optional
from c8ydm.framework.smartrest import SmartRESTMessage
from c8ydm.framework.modulebase import Listener
from c8ydm.core.shell import CommandAlias, CommandAliasWithArgs, InvalidCommandError, CommandFailedError, CommandTimeoutError, TimeoutExpired
class CommandHandler(Listener):

    logger = logging.getLogger(__name__)
    fragment = 'c8y_Command'
    command_message_id = '511'
    _supported_commands = None
    timeout = 60

    def _set_executing(self):
        executing = SmartRESTMessage('s/us', '501', [self.fragment])
        self.agent.publishMessage(executing)

    def _set_success(self):
        success = SmartRESTMessage('s/us', '503', [self.fragment])
        self.agent.publishMessage(success)
    
    def _set_success_with_result(self, result):
        success = SmartRESTMessage('s/us', '503', [self.fragment, result])
        self.agent.publishMessage(success)

    def _set_failed(self, reason):
        failed = SmartRESTMessage('s/us', '502', [self.fragment, reason])
        self.agent.publishMessage(failed)

    def _get_supported_commands(self) -> List[CommandAlias]:
        """Convert a command alias to its command equalivalent. If not match is found
        then None will be returned.

        Returns:
            Optional[List[CommandAlias]]: Command resolved by the alias
        """
        # Base commands
        commands = [
            CommandAlias('show packages', 'dpkg-query --show'),
            CommandAlias('show processes', 'pgrep -fa \'.+\''),
            CommandAlias('show memory', 'cat /proc/meminfo'),
            CommandAlias('show cpu-usage', 'top -b -n 2'),
            CommandAlias('show disk-usage', 'df -h'),
            CommandAlias('show agent logs', 'cat ~/.cumulocity/agent.log'),
            CommandAlias('remove apt-lists', 'rm -rf /var/lib/apt/lists/*'),
            CommandAliasWithArgs(
                r'show logs (\w+) ?(ERROR|WARN|INFO|DEBUG)?',
                'journalctl -u \\1 -n 100 | grep "\\2"',
                usage='show logs <name> [ERROR|WARN|INFO|DEBUG]'),
        ]

        linux_commands = [
            CommandAlias('show uptime', 'uptime'),
            CommandAlias('show uptime since', 'uptime --since'),
            CommandAlias('show uptime seconds',
                         'awk \'{print $1}\' /proc/uptime'),
            CommandAlias('exec reboot', 'shutdown -r +10'),
        ]
        commands.extend(linux_commands)
        return commands

    def handleOperation(self, message):
        """Callback that is executed for any operation received

            Raises:
            Exception: Error when handling the operation
        """
        if 's/ds' in message.topic and self.command_message_id == message.messageId:
            try:
                self._supported_commands = self._get_supported_commands()
                self.logger.info(f'Supported Commands {self._supported_commands}')
                messages = message.values
                self._set_executing()
                self.logger.info(f'Shell Command Message received: {messages}')

                # Parse command
                raw_cmd = re.sub(r';?\s*\n', '; ', ';'.join(message.values[1:]))
                raw_cmd = re.sub('^"(.*)"$', '\\1', raw_cmd)
                # replace escaped double quote, with literal quote
                raw_cmd = raw_cmd.replace(r'\"', '"')

                # Check for help
                if raw_cmd == 'show help':
                    logging.info(f'Showing shell help')
                   
                    self._set_success_with_result('\n'.join(self._show_help()))
                    return

                resolved_cmd = self._resolve_command(raw_cmd)

                if resolved_cmd:
                    logging.info(f'Using pre-defined command')
                elif self.check_command(raw_cmd):
                    logging.info(f'Using extended command')
                    resolved_cmd = CommandAlias(raw_cmd, raw_cmd)

                if not resolved_cmd:
                    raise InvalidCommandError().add_context(f'command: {raw_cmd}')

                _, output_text = resolved_cmd.execute_command(
                    raw_cmd, timeout=60)
                if sys.getsizeof(output_text) > 16000:
                    self._set_failed(f'Output of command exceeds 16 KB Payload Limit')
                else:
                    self._set_success_with_result(output_text)

            except (InvalidCommandError, CommandFailedError, CommandTimeoutError) as ex:
                logging.error(f'Command error. Exception={ex}')
                self._set_failed(f'{ex}')
            except TimeoutExpired as ex:
                self._set_failed(f'{ex}')
            except Exception as ex:
                logging.error(f'Handling operation error. exception={ex}')
                self._set_failed(f'Unhandled exception. exception={ex}')

    def _resolve_command(self, user_input: str) -> Optional[CommandAlias]:
        """Convert a command alias to its command equalivalent. If not match is found
        then None will be returned.

        Args:
            user_input (str): User input command

        Returns:
            Optional[str]: Command resolved by the alias
        """
        for cmd in self._supported_commands:
            if cmd.is_match(user_input):
                return cmd
        return None
    
    def _show_help(self) -> List[str]:
        """Get the list of usage of the supported commands

        Returns:
            List[str]: List of command usages
        """
        return [cmd.show_usage()
                for cmd in self._supported_commands]

    def _resolve_command(self, user_input: str) -> Optional[CommandAlias]:
        """Convert a command alias to its command equalivalent. If not match is found
        then None will be returned.

        Args:
            user_input (str): User input command

        Returns:
            Optional[str]: Command resolved by the alias
        """
        for cmd in self._supported_commands:
            if cmd.is_match(user_input):
                return cmd
        return None

    @classmethod
    def check_command(cls, cmd: str) -> bool:
        """Check command if it is valid and does not use any forbidden/dangerous commands

        Args:
            cmd (str): command

        Returns:
            bool: True if the command can be run or not
        """
        # allowed_commands = ['cat', 'grep', 'ls', 'uptime']
        forbidden_commands = [
            'alias',
            'nano',
            'reboot',
            'shutdown',
            'vim'
        ]

        forbidden_symbols = [
            '\\e',
            '$',
            '<',
        ]

        forbidden_usage = []
        for item in forbidden_commands:
            match = re.search(fr'\b{item}\s+', cmd)
            match_end = re.search(fr'\b{item}$', cmd)
            if match is not None or match_end is not None:
                logging.warning(
                    f'Detected forbidden command: name={item}, match={match}')
                forbidden_usage.append(item)

        for symbol in forbidden_symbols:
            if symbol in cmd:
                logging.warning(
                    f'Detected forbidden command: name={item}, symbol={symbol}')
                forbidden_usage.append(cmd)

        return len(forbidden_usage) == 0

    def getSupportedOperations(self):
        return [self.fragment]

    def getSupportedTemplates(self):
        return []


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""  
Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors

SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import logging
import time
import certifi

import paho.mqtt.client as mqtt


class Bootstrap():
    bootstrapped = False

    def __init__(self, serial, path, configuration):
        self.logger = logging.getLogger(__name__)
        self.serial = serial
        self.configuration = configuration
        self.url = self.configuration.getValue('mqtt', 'url')
        self.port = self.configuration.getValue('mqtt', 'port')
        self.ping = self.configuration.getValue('mqtt', 'ping.interval.seconds')
        self.tls = self.configuration.getBooleanValue('mqtt', 'tls')
        #self.cacert = self.configuration.getValue('mqtt', 'cacert') 
        self.terminated = False

    def on_connect(self, client, userdata, flags, rc):
        self.logger.debug('Bootstrap connected with result code: ' + str(rc))
        client.subscribe('s/dcr')

    def on_disconnect(self, client, userdata, rc):
        self.logger.debug('Bootstrap disconnected with result code: ' + str(rc))

    def on_messageRegistration(self, client, userdata, msg):
        message = msg.payload.decode('utf-8')
        messageParts = message.split(',')
        self.logger.debug(messageParts)
        if messageParts[0] == '70':

            while not self.bootstrapped:
                try:
                    self.logger.debug('Storing credentials...')
                    self.configuration.writeCredentials(messageParts[1], messageParts[2], messageParts[3])
                    self.logger.debug('Storing credentials successful')
                    client.unsubscribe('s/dcr')
                    self.bootstrapped = True
                except Exception as e:
                    self.logger.debug('Storing credentials failed. Waiting 5 Sec. to retry..')
                    self.logger.exception(e)
                    time.sleep(5)

    def bootstrap(self):
        self.logger.info('Start bootstrap client')

        client = mqtt.Client(client_id=self.serial)
        client.on_message = self.on_messageRegistration
        client.on_connect = self.on_connect
        client.on_disconnect = self.on_disconnect

        credentials = self.configuration.getBootstrapCredentials()
        if self.tls:
             client.tls_set(certifi.where())
        client.username_pw_set(credentials[0] + '/' + credentials[1], credentials[2])
        client.connect(self.url, int(self.port), int(self.ping))

        client.loop_start()
        client.publish('s/ucr')

        while not self.bootstrapped and not self.terminated:
            time.sleep(5)
            self.logger.debug('poll credentials')
            client.publish('s/ucr')

        client.on_message = None
        client.on_connect = None
        client.disconnect()
        client.loop_stop()
        client = None
        time.sleep(5)
        self.logger.info('Stop bootstrap client')

    def stop(self):
        self.terminated = True
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors

SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import logging
import subprocess

class GeoPositionResolver:
    logger = logging.getLogger(__name__)

    def get_pos_by_ip(self, ip):
        lat_lng = {}
        try:
            if ip:
                self.logger.info(f'Getting GEO Data for IP {ip}...')
                geo_ip_data = subprocess.Popen(['geoiplookup', ip], stdout=subprocess.PIPE)
                output = geo_ip_data.stdout.read().decode('utf-8')
                if output:
                    self.logger.debug(f'Output of geoiplookup {output}')
                    geo_data_list = output.split('\n')
                    for line in geo_data_list:
                        if "GeoIP City Edition" in line:
                            geo_pos_list = line.split(', ')
                            self.logger.debug(f'list of Geo Data {geo_pos_list}')
                            lat_lng['lat'] = geo_pos_list[6]
                            lat_lng['lng'] = geo_pos_list[7]
            return lat_lng
        except Exception as ex:
            self.logger.error(f'Error on retrieving GEO Position Data from IP: {ex}')
            return lat_lng
